'From Squeak3.5 of ''11 April 2003'' [latest update: #5180] on 5 June 2003 at 11:38:19 am'!

!InterpreterSimulator methodsFor: 'image save/restore' stamp: 'crl 6/5/2003 11:08'!
writeImageFileIO: numberOfBytesToWrite
	"Actually emit the first numberOfBytesToWrite object memory bytes onto the snapshot."

	| headerSize file |

	headerSize _ 64.

	[
		file _ (FileStream fileNamed: imageName) binary.
		file == nil ifTrue: [^nil].
	
		{
			self imageFormatVersion.
			headerSize.
			numberOfBytesToWrite.
			self startOfMemory.
			specialObjectsOop.
			lastHash.
			self ioScreenSize.
			fullScreenFlag.
			extraVMMemory
		}
			do: [:long | self putLong: long toFile: file].
	
		"Pad the rest of the header."
		7 timesRepeat: [self putLong: 0 toFile: file].
	
		"Position the file after the header."
		file position: headerSize.
	
		"Write the object memory."
		1
			to: numberOfBytesToWrite // 4
			do: [:index |
				self
					putLong: (memory at: index)
					toFile: file].
	
		self success: true
	]
		ensure: [file close]! !

!InterpreterSimulator methodsFor: 'initialization' stamp: 'crl 6/5/2003 11:03'!
openOn: fileName extraMemory: extraBytes
	"InterpreterSimulator new openOn: 'clone.im' extraMemory: 100000"

	| f version headerSize count oldBaseAddr bytesToShift swapBytes |
	"open image file and read the header"
[
	f _ FileStream readOnlyFileNamed: fileName.
	imageName _ f fullName.
	f binary.
	version _ self nextLongFrom: f.  "current version: 16r1966 (=6502)"
	(self readableFormat: version)
		ifTrue: [swapBytes _ false]
		ifFalse: [(version _ self byteSwapped: version) = self imageFormatVersion
					ifTrue: [swapBytes _ true]
					ifFalse: [self error: 'incomaptible image format']].
	headerSize _ self nextLongFrom: f swap: swapBytes.
	endOfMemory _ self nextLongFrom: f swap: swapBytes.  "first unused location in heap"
	oldBaseAddr _ self nextLongFrom: f swap: swapBytes.  "object memory base address of image"
	specialObjectsOop _ self nextLongFrom: f swap: swapBytes.
	lastHash _ self nextLongFrom: f swap: swapBytes.  "Should be loaded from, and saved to the image header"
	"savedWindowSize _ self nextLongFrom: f swap: swapBytes."
	lastHash = 0 ifTrue: [lastHash _ 999].

	savedWindowSize	_ self nextLongFrom: f swap: swapBytes.
	fullScreenFlag		_ self nextLongFrom: f swap: swapBytes.
	extraVMMemory		_ self nextLongFrom: f swap: swapBytes.

	"allocate interpreter memory"
	memoryLimit _ endOfMemory + extraBytes.

	"read in the image in bulk, then swap the bytes if necessary"
	f position: headerSize.
	memory _ Bitmap new: memoryLimit // 4.
	count _ f readInto: memory startingAt: 1 count: endOfMemory // 4.
	count ~= (endOfMemory // 4) ifTrue: [self halt].
	] ensure: [f close].
	swapBytes ifTrue: [Utilities informUser: 'Swapping bytes of foreign image...'
								during: [self reverseBytesInImage]].

	self initialize.
	bytesToShift _ 0 - oldBaseAddr.  "adjust pointers for zero base address"
	endOfMemory _ endOfMemory.
	Utilities informUser: 'Relocating object pointers...'
				during: [self initializeInterpreter: bytesToShift].
! !

!InterpreterSimulator methodsFor: 'memory access' stamp: 'crl 3/3/2003 01:15'!
cCoerce: value to: cTypeString
	"Type coercion for translation only; just return the value when running in Smalltalk."

	^value == nil
		ifTrue: [value]
		ifFalse: [value coerceTo: cTypeString sim: self]! !


!InterpreterSimulatorLSB methodsFor: 'image save/restore' stamp: 'crl 2/28/2003 01:22'!
putLong: n toFile: f
	"Append the given 4-byte long word to the given file in my byte order. (Bytes will be swapped, if necessary, when the image is read on a different platform.) Set successFlag to false if the write fails."

	| remainingValue |

	remainingValue _ n.
	4 timesRepeat: [
		f nextPut: (remainingValue bitAnd: 16rFF).
		remainingValue _ remainingValue bitShift: -8].

	self success: true! !


!InterpreterSimulatorMSB methodsFor: 'image save/restore' stamp: 'crl 6/5/2003 11:37'!
putLong: n toFile: f
	"Append the given 4-byte long word to the given file in my byte order. (Bytes will be swapped, if necessary, when the image is read on a different platform.) Set successFlag to false if the write fails."

	f
		nextPut: (n bitShift: -24);
		nextPut: ((n bitAnd: 16rFF0000) bitShift: -16);
		nextPut: ((n bitAnd: 16rFF00) bitShift: -8);
		nextPut: (n bitAnd: 16rFF).

	self success: true! !


!InterpreterSimulatorLSB reorganize!
('debug support' charsOfLong: displayForm:)
('initialization' nextLongFrom:)
('memory access' byteAt: byteAt:put: halfWordAt: halfWordAt:put:)
('image save/restore' putLong:toFile:)
!


!InterpreterSimulator reorganize!
('I/O primitives' clipboardRead:Into:At: clipboardSize clipboardWrite:From:At: ioGetNextEvent: ioHasDisplayDepth: ioLowResMSecs ioProcessEvents ioScreenDepth ioSetInputSemaphore: primitiveBeDisplay primitiveKbdNext primitiveKbdPeek primitiveMouseButtons primitiveMousePoint primitiveScreenSize showDisplayBits:w:h:d:left:right:top:bottom:)
('I/O primitives support' copyBits displayLocation drawLoopX:Y: ioMSecs ioMicroMSecs ioProcessEventsEveryMSecs: ioScreenSize ioSeconds primitiveRelinquishProcessor sizeof: warpBits)
('bytecode routines' pushLiteralConstantBytecode pushLiteralVariableBytecode pushReceiverVariableBytecode pushTemporaryVariableBytecode storeAndPopReceiverVariableBytecode storeAndPopTemporaryVariableBytecode)
('debug printing' cr print: printChar: printNum:)
('debug support' charsOfLong: classAndSelectorOfMethod:forReceiver: compactClassAt: dumpHeader: dumpMethodHeader: fullGC headerStart: hexDump100: hexDump: longPrint: nameOfClass: printStack shortPrint: stringOf:)
('file primitives' fileValueOf: makeDirEntryName:size:createDate:modDate:isDir:fileSize: primitiveDirectoryLookup primitiveFileDelete primitiveFileOpen primitiveFileRename primitiveImageName sqFile:Read:Into:At: sqFile:SetPosition: sqFile:Truncate: sqFile:Write:From:At: sqFileAtEnd: sqFileClose: sqFileFlush: sqFileGetPosition: sqFileSize: vmPathGet:Length: vmPathSize)
('float primitives' fetchFloatAt:into: storeFloatAt:from:)
('image save/restore' writeImageFileIO:)
('initialization' close initialize nextLongFrom: nextLongFrom:swap: openOn: openOn:extraMemory: reverseBytesFrom:to: startOfMemory)
('interpreter shell' dispatchOn:in: fetchByte isIntegerValue:)
('memory access' byteAt: byteAt:put: cCoerce:to: halfWordAt: halfWordAt:put: integerAt: integerAt:put: longAt: longAt:put: sqGrowMemory:By: sqMemoryExtraBytesLeft: sqShrinkMemory:By:)
('other primitives' ioForceDisplayUpdate primBitmapcompresstoByteArray primBitmapdecompressfromByteArrayat primStringcomparewithcollated primStringfindSubstringinstartingAtmatchTable primStringindexOfAsciiinStringstartingAt primStringtranslatefromtotable primitiveBeep primitiveGetAttribute)
('plugin support' callExternalPrimitive: classNameOf:Is: flushExternalPrimitives ioLoadExternalFunction:OfLength:FromModule:OfLength: ioLoadFunction:From: loadNewPlugin: sqGetInterpreterProxy)
('security' ioCanRenameImage ioCanWriteImage)
('testing' findNewMethodInClass: ioExit isInterpreterProxy objectBefore: profile: profileSends: runForNBytes: stackDepth stats test testBecome validOop: validate validate: validateActiveContext validateOopsIn:)
!

